# 
# Copyright (C) 2004,2007 VRTech Industrial Technologies - www.vrtech.com.br.
# All Rights Reserved.
# This code is published under the Common Public License.
# 
# $Id$
# Authors: Rafael de Pelegrini Soares
#

Import('env')
import os
import os.path
import sys
import string
import SCons.Util

#===========================================================================
# Installation prefix
env['prefix'] = '/usr/local'

#===========================================================================
# Installation with permitions
import SCons
from SCons.Script.SConscript import SConsEnvironment
SConsEnvironment.Chmod = SCons.Action.ActionFactory(os.chmod,
        lambda dest, mode: 'Chmod("%s", 0%o)' % (dest, mode))
def InstallPerm(env, dest, files, perm):
    obj = env.Install(dest, files)
    for i in obj:
        env.AddPostAction(i, env.Chmod(str(i), perm))
    return obj
SConsEnvironment.InstallPerm = InstallPerm

#============================================================================
# Cross compiling stuff
#
target = env['platform']
host = env['host']
if not host == target:
        cross = False
        if target == 'win32':
                # cross compiling with mingw
		prefix = 'i586-mingw32';
		path = env['ENV']['PATH'] + os.pathsep + '/usr/'+prefix+'/bin'
                mingw = env.WhereIs(prefix+'-gcc', path )
		if not mingw:
			prefix = 'i586-mingw32msvc';
			path = env['ENV']['PATH'] + os.pathsep + '/usr/'+prefix+'/bin'
			mingw = env.WhereIs(prefix+'-gcc', path)
                if mingw:
                        env.Tool('mingw')
                        env['ENV']['PATH'] = path
			env['prefix'] = os.path.join('/usr/local', prefix)
			env.Append(CPPPATH = '/usr/local/'+prefix+'/include')
			env.Append(LIBPATH = '/usr/local/'+prefix+'/lib')
                        # a few things differ from std mingw:
                        env['F77'] = 'g77'
                        tools = ['CC', 'CXX', 'AS', 'RC', 'AR', 'RANLIB', 'F77']
                        for i in tools:
				env[i] = prefix + '-' + env[i]
                        env['PROGSUFFIX'] = '.exe'
                        env['SHLIBSUFFIX'] = '.dll'
                        env['SHLIBPREFIX'] = ''
                        cross = True
	# FIXME: add other cross compiling tools
	if not cross:
		print '*** error: cross compilation for given target unavailable'
		Exit(1)
if target == 'win32':
    if host == 'win32':
        env.Tool('mingw')
    env.Append(LINKFLAGS = ['-Wl,--enable-runtime-pseudo-reloc', '-Wl,--enable-auto-import'])
    if env['configuration'] == 'release':
	    env.Append(LINKFLAGS = ['-s', '-mwindows'])



#==================================================================
# Mode defines: debug, release, flags, etc
#
if env['configuration'] == 'debug' :
	env.Append(CPPDEFINES= ['DEBUG', '_DEBUG'] )
	env.Append(CCFLAGS='-g')
	env.Append(F77FLAGS='-g')
	env.Append(SHF77FLAGS='-g')
if env['configuration'] == 'coverage' :
	env.Append(CCFLAGS='-g')
	env.Append(F77FLAGS='-g')
	env.Append(SHF77FLAGS='-g')
if env['configuration'] == 'release' :
	env.Append(CPPDEFINES= ['NDEBUG'] )
	env.Append(CCFLAGS='-O3')
	env.Append(F77FLAGS='-O3')

# Eneble exceptions for C code
env.Append(CCFLAGS = ['-fexceptions'])
env.Append(F77FLAGS = ['-fexceptions'])
env.Append(SHF77FLAGS = ['-fexceptions'])


